package	jp.co.sds_corp.report;

import jp.co.sds_corp.report.source.ISource;
import jp.co.sds_corp.report.source.SourceFactory;
import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.script.IScriptParser;
import jp.co.sds_corp.util.script.ScriptParserFactory;

/**
{@link IReport}を生成するFactoryクラスです。
以下の形式のXMLファイルの、typeによって{@link IReport}を生成します。
<pre>
&lt;Report
	name="ReportTest"
	type="Excel"
	template="${Report.dir}/table.xls"
	sheet="Sheet1"&gt;
	&lt;Source .../&gt;
	&lt;Script .../&gt;
&lt;/Report&gt;
</pre>
typeの代わりにclass属性が指定された場合は、指定されたオブジェクトを生成します。
&gt;Report
    class="<EM>foo.bar.class</EM>"&gt;
&lt;/Report&gt;
</pre>
*/
public class ReportFactory
{
	/**
	* 設定情報により、{@link IReport}を生成します。
	* @param    config  設定情報
	* @return   生成された{@link IReport}
	*/
	public static IReport createReport( IConfiguration config )
		throws Exception
	{
		return createReport( config, null );
	}

	/**
	* 設定情報により、{@link IReport}を生成します。
	* @param    config  設定情報
	* @param    prop    properties
	* @return   生成された{@link IReport}
	*/
	public static IReport createReport( IConfiguration config, CmnProperty prop )
		throws Exception
	{
		IReport ret = null;
		String type = config.getAttributes().get( "type", null );
		String cls = config.getAttributes().get( "class", null );
		if ( type != null ) {
			if ( type.equals( "Excel" )) {
				ret = new ExcelReport();
			} else if ( type.equals( "Jxl" )) {
				ret = new JxlReport();
			} else if ( type.equals( "PDF" )) {
				ret = new ExcelPDFReport();
			} else if ( type.equals( "PDF1" )) {
				ret = new ExcelPDFReport1();
			} else if ( type.equals( "PDF2" )) {
				ret = new ExcelPDFReport2();
			} else {
				throw( new IllegalArgumentException( "illegal type.[" + type + "]" ));
			}
		} else if ( cls != null ) {
			ret = (IReport )Class.forName( cls ).newInstance();
		}
		if ( ret == null ) {
			throw( new IllegalArgumentException( "type/class not defined." ));
		}

		ret.setProperty( prop );
		ret.init( config );
		IConfiguration c;
		c = config.getChild( "Script" );
		if ( c != null ) {
			IScriptParser parser = ScriptParserFactory.createParser( c, prop );
			ret.setScriptParser( parser );
		}
		c = config.getChild( "Source" );
		if ( c != null ) {
			ISource source = SourceFactory.createSource( c, prop );
			ret.setSource( source );
		}
		ret.setAttribute( "Report", ret );
		return ret;
	}

}
